using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Soap;

namespace SoapSerializer
{
	/// <summary>
	/// Summary description for SoapSerializerTester.
	/// </summary>
	class SoapSerializerTester
	{
		bool Serialize(string strPath, Object objStuff)
		{
			bool bReturn = true;
			try
			{
				// Create a file stream
				FileStream fstmOutput = new FileStream(strPath,FileMode.Create);

				// Create the formatter we'll associate
				// with this stream
				SoapFormatter sfmtFormatter = new SoapFormatter();

				// Serialize and save the object
				sfmtFormatter.Serialize(fstmOutput,objStuff);

				// Close the file stream
				fstmOutput.Close();
			} //try
			catch(Exception e)
			{
				Console.WriteLine("{0}",e.Message);
				bReturn = false;
			} // catch

			return bReturn;
		}

		Object Deserialize(string strPath)
		{
			Object objReturn = null;
			try
			{
				// Create a file stream
				FileStream fstmInput = new FileStream(strPath,FileMode.Open);

				// Create the formatter we'll associate
				// with this stream
				SoapFormatter sfmtFormatter = new SoapFormatter();

				// Serialize and save the object
				objReturn = sfmtFormatter.Deserialize(fstmInput);

				// Close the file stream
				fstmInput.Close();
			} //try
			catch(Exception e)
			{
				Console.WriteLine("{0}",e.Message);
				objReturn = null;
			} // catch

			return objReturn;
		}

		static void Main(string[] args)
		{
			try
			{
				// Create an instance...
				SoapSerializerTester sst1 = new SoapSerializerTester();

				// Create a jagged array to serialize/
				// deserialize
				int[][] q = {new int[2], new int[4], new int[3]};
				q[0][0] = 4;
				q[0][1] = 7;
				q[1][0] = 15;
				q[1][1] = 72;
				q[1][2] = 6;
				q[1][3] = 167;
				q[2][0] = 1;
				q[2][1] = 90;
				q[2][2] = 659;

				// Serialize the jagged array
				if ( !sst1.Serialize("c:\\out.txt",(Object)q) )
				{
					Console.WriteLine("Failed to serialize object...");
					return;
				} // if
				 
				int[][] r = (int[][])sst1.Deserialize("c:\\out.txt");
				if ( (Object)r == null )
				{
					Console.WriteLine("Failed to deserialize object...");
					return;
				} // if
		
				// Write results...
				Console.WriteLine("Result was {0}",r.ToString());
				Console.WriteLine(" [0][0] = {0}",r[0][0]);
				Console.WriteLine(" [0][1] = {0}",r[0][1]);
				Console.WriteLine(" [1][0] = {0}",r[1][0]);
				Console.WriteLine(" [1][1] = {0}",r[1][1]);
				Console.WriteLine(" [1][2] = {0}",r[1][2]);
				Console.WriteLine(" [1][3] = {0}",r[1][3]);
				Console.WriteLine(" [2][0] = {0}",r[2][0]);
				Console.WriteLine(" [2][1] = {0}",r[2][1]);
				Console.WriteLine(" [2][2] = {0}",r[2][2]);
				Console.ReadLine();
			} // try
			catch (Exception e)
			{
				Console.WriteLine("{0}",e.Message);
			} // catch
		}
	}
}
