using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;
using System.Xml.Serialization;

namespace ParmEncoder
{
	[AttributeUsage(AttributeTargets.Method)]
	public class SoapDumperAttribute : SoapExtensionAttribute 
	{
		private string m_strFilename = "C:\\SoapLog.txt";
		private int m_iPriority;

		public override Type ExtensionType 
		{
			get
			{
				return typeof(SoapDumper);
			}
		}

		public override int Priority 
		{
			get
			{
				return m_iPriority;
			}
			set
			{
				m_iPriority = value;
			}
		}

		public string LogFile 
		{
			get 
			{
				return m_strFilename;
			}
			set 
			{
				m_strFilename = value;
			}
		}
	}

	public class SoapDumper : SoapExtension 
	{
		private Stream m_stmOld;
		private Stream m_stmNew;
		private string m_strLogFile;

		public override object GetInitializer(Type serviceType) 
		{
			return GetType();
		}

		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute) 
		{
			return ((SoapDumperAttribute)attribute).LogFile;
		}

		public override void Initialize(object initializer) 
		{
			m_strLogFile = (string) initializer;
		}

		public override void ProcessMessage(SoapMessage message) 
		{
			switch (message.Stage) 
			{
				case SoapMessageStage.BeforeSerialize:
					break;

				case SoapMessageStage.AfterSerialize:
					LogOutput(message);
					break;

				case SoapMessageStage.BeforeDeserialize:
					LogInput(message);
					break;

				case SoapMessageStage.AfterDeserialize:
					break;

				default:
					throw new Exception("invalid stage");
			}
		}

		public override Stream ChainStream( Stream stream )
		{
			m_stmOld = stream;
			m_stmNew = new MemoryStream();
			return m_stmNew;
		}

		public void LogOutput( SoapMessage message )
		{
			m_stmNew.Position = 0;
			FileStream fstmLog = new FileStream(m_strLogFile, FileMode.Append, FileAccess.Write);
			StreamWriter stmLog = new StreamWriter(fstmLog);
			stmLog.WriteLine("---------------------------------- SOAP Response at " + DateTime.Now);
			stmLog.Flush();
			Copy(m_stmNew,fstmLog);
			fstmLog.Close();
			m_stmNew.Position = 0;
			Copy(m_stmNew,m_stmOld);
		}

		public void LogInput( SoapMessage message )
		{
			Copy(m_stmOld,m_stmNew);
			FileStream fstmLog = new FileStream(m_strLogFile, FileMode.Append, FileAccess.Write);
			StreamWriter stmLog = new StreamWriter(fstmLog);
			stmLog.WriteLine("================================== SOAP Request at " + DateTime.Now);
			stmLog.Flush();
			m_stmNew.Position = 0;
			Copy(m_stmNew,fstmLog);
			fstmLog.Close();
			m_stmNew.Position = 0;
		}

		void Copy(Stream stmFrom, Stream stmTo)
		{
			TextReader trFrom = new StreamReader(stmFrom);
			TextWriter twTo = new StreamWriter(stmTo);
			twTo.WriteLine(trFrom.ReadToEnd());
			twTo.Flush();
		}
	}

	/// <summary>
	/// Summary description for Service1.
	/// </summary>
	public class Service1 : System.Web.Services.WebService
	{
		public Service1()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}
		#endregion

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
		}

		public struct Point
		{
			public int x;
			public int y;
		}

		[SoapDocumentMethodAttribute("http://www.myurl.com/wrapparms",
			 Use=System.Web.Services.Description.SoapBindingUse.Literal,
			 ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
		[WebMethod]
		[SoapDumper()]
		public string WrapParms(string strInput, Point pt)
		{
			string strReturn = string.Format("The input string was '{0}', and the point values were {1} and {2}",strInput,pt.x,pt.y);
			return strReturn;
		}

		[SoapDocumentMethodAttribute("http://www.myurl.com/wrapparms2",
			 RequestNamespace="http://www.myurl.com/wrapparms2",
			 ResponseNamespace="http://www.myurl.com/wrapparms2",
			 Use=System.Web.Services.Description.SoapBindingUse.Literal,
			 ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
		[WebMethod]
		[SoapDumper()]
		public string WrapParms2(
			[System.Xml.Serialization.XmlElementAttribute(Namespace="http://www.myurl.com/wrapparms2", IsNullable=false)]string strInput,
			[System.Xml.Serialization.XmlElementAttribute(Namespace="http://www.myurl.com/wrapparms2", IsNullable=false)]Point pt)
		{
			string strReturn = string.Format("The input string was '{0}', and the point values were {1} and {2}",strInput,pt.x,pt.y);
			return strReturn;
		}

		[SoapDocumentMethodAttribute("http://www.myurl.com/bareparms",
			 Use=System.Web.Services.Description.SoapBindingUse.Literal,
			 ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Bare)]
		[WebMethod]
		[SoapDumper()]
		public string BareParms(string strInput, Point pt)
		{
			string strReturn = string.Format("The input string was '{0}', and the point values were {1} and {2}",strInput,pt.x,pt.y);
			return strReturn;
		}

		[SoapDocumentMethodAttribute("http://www.myurl.com/bareparms2",
			 RequestNamespace="http://www.myurl.com/bareparms2",
			 ResponseNamespace="http://www.myurl.com/bareparms2",
			 Use=System.Web.Services.Description.SoapBindingUse.Literal,
			 ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Bare)]
		[WebMethod]
		[SoapDumper()]
		public string BareParms2(
			[System.Xml.Serialization.XmlElementAttribute(Namespace="http://www.myurl.com/bareparms2", IsNullable=false)]string strInput,
			[System.Xml.Serialization.XmlElementAttribute(Namespace="http://www.myurl.com/bareparms2", IsNullable=false)]Point pt)
		{
			string strReturn = string.Format("The input string was '{0}', and the point values were {1} and {2}",strInput,pt.x,pt.y);
			return strReturn;
		}

		[SoapDocumentMethodAttribute("http://www.myurl.com/renameparms",
			 Use=System.Web.Services.Description.SoapBindingUse.Literal,
			 ParameterStyle=System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
		[WebMethod]
		[SoapDumper()]
		[return: XmlElement("MyUniqueReturnValueElementName",IsNullable=false)]
		public string RenameParms(
			[XmlElement("InputString")] string strInput, // normally "strInput"
			[XmlElement("InputPoint")] Point pt) // normally "pt"
		{
			string strReturn = string.Format("The input string was '{0}', and the point values were {1} and {2}",strInput,pt.x,pt.y);
			return strReturn;
		}
	}
}
