using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Security;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace Ch7CSEx4
{
	[AttributeUsage(AttributeTargets.Method)]
	public class SoapDecrypterAttribute : SoapExtensionAttribute 
	{
		private string m_strFilename = "C:\\DecryptLog.txt";
		private int m_iPriority;

		public override Type ExtensionType 
		{
			get
			{
				return typeof(SoapDecrypter);
			}
		}

		public override int Priority 
		{
			get
			{
				return m_iPriority;
			}
			set
			{
				m_iPriority = value;
			}
		}

		public string LogFile 
		{
			get 
			{
				return m_strFilename;
			}
			set 
			{
				m_strFilename = value;
			}
		}
	}

	public class SoapDecrypter : SoapExtension 
	{
		private Stream m_stmOld;
		private Stream m_stmNew;
		private string m_strLogFile;
		private byte[] m_key;
		private byte[] m_iv;

		public SoapDecrypter()
		{
			// Create the DES key...note this and the initialization
			// vector must be the same as that used by the client
			// or the value cannot be decrypted...
			m_key = new byte[8];
			m_key[0] = 104; // random data, but same as client
			m_key[1] = 91;
			m_key[2] = 2;
			m_key[3] = 56;
			m_key[4] = 226;
			m_key[5] = 77;
			m_key[6] = 165;
			m_key[7] = 194;

			// Create the DES initialization vector
			m_iv = new byte[8];
			m_iv[0] = 69;
			m_iv[1] = 148;
			m_iv[2] = 67;
			m_iv[3] = 76;
			m_iv[4] = 139;
			m_iv[5] = 21;
			m_iv[6] = 89;
			m_iv[7] = 247;
		}

		public override object GetInitializer(Type serviceType) 
		{
			return GetType();
		}

		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute) 
		{
			return ((SoapDecrypterAttribute)attribute).LogFile;
		}

		public override void Initialize(object initializer) 
		{
			m_strLogFile = (string) initializer;
		}

		public override void ProcessMessage(SoapMessage message) 
		{
			switch (message.Stage) 
			{
				case SoapMessageStage.BeforeSerialize:
					break;

				case SoapMessageStage.AfterSerialize:
					LogOutput(message);
					break;

				case SoapMessageStage.BeforeDeserialize:
					DecryptInput(message);
					break;

				case SoapMessageStage.AfterDeserialize:
					break;

				default:
					throw new Exception("invalid stage");
			}
		}

		public override Stream ChainStream( Stream stream )
		{
			m_stmOld = stream;
			m_stmNew = new MemoryStream();
			return m_stmNew;
		}

		public void LogOutput( SoapMessage message )
		{
			// Output logging is the same as for previous
			// SoapExtension (as seen in Chapter 6)
			m_stmNew.Position = 0;
			FileStream fstmLog = new FileStream(m_strLogFile, FileMode.Append, FileAccess.Write);
			StreamWriter stmLog = new StreamWriter(fstmLog);
			stmLog.WriteLine("---------------------------------- SOAP Response at " + DateTime.Now);
			stmLog.Flush();
			Copy(m_stmNew,fstmLog);
			fstmLog.Close();
			m_stmNew.Position = 0;
			Copy(m_stmNew,m_stmOld);
		}

		public void DecryptInput( SoapMessage message )
		{
            // Make a copy of the incoming SOAP stream
			Copy(m_stmOld,m_stmNew);

			// Create a file to contain the logged output
			FileStream fstmLog = new FileStream(m_strLogFile, FileMode.Append, FileAccess.Write);
			StreamWriter stmLog = new StreamWriter(fstmLog);
			stmLog.WriteLine("================================== (Encrypted) SOAP Request at " + DateTime.Now);
			stmLog.Flush();

			// Copy the SOAP packet as it came in
			m_stmNew.Position = 0;
			Copy(m_stmNew,fstmLog);
            fstmLog.Close();

			// Reset the SOAP stream to load into XPath processor
			m_stmNew.Position = 0;

            try
            {
                // Load it and create a navigator
                XmlDocument xdc = new XmlDocument();
                xdc.Load(m_stmNew);

                // Query for the encrypted parameter.
                XmlNamespaceManager mgr = new XmlNamespaceManager(xdc.NameTable);
                mgr.AddNamespace("soap","http://schemas.xmlsoap.org/soap/envelope/");
                XmlNode nodeBody = xdc.SelectSingleNode("/soap:Envelope/soap:Body",mgr);
                XmlNode nodeParm = ((XmlNode)nodeBody.FirstChild).FirstChild;

                // Pull element's encrypted value as a string
                string strEncrypted = nodeParm.InnerText;

                // Convert the string to a byte array
                byte[] bytIn = new byte[4096];
                byte[] bytEnc = Convert.FromBase64String(strEncrypted);
                bytEnc.CopyTo(bytIn,0);
      
                // Create streams for decryption
                byte[] bytOut = new byte[4096];
                MemoryStream stmOut = new MemoryStream(bytOut);

                // Create the service provider
                DES des = new DESCryptoServiceProvider();

                // Create the crypto stream and buffer
                CryptoStream stmDecrypt = new CryptoStream(stmOut,des.CreateDecryptor(m_key,m_iv),CryptoStreamMode.Write);

                // Encrypt to memory stream
                stmDecrypt.Write(bytIn,0,bytEnc.Length);
                stmDecrypt.FlushFinalBlock();

                // Write XML
                UTF8Encoding enc = new UTF8Encoding();
                nodeParm.InnerText = enc.GetString(bytOut,0,(int)stmOut.Position);

                // Write decrypted output to stream. The stream is
                // truncated because the Base64 representation of
                // the parameter is textually longer than the
                // decrypted length, so we would have garbage
                // characters left at the end of the stream...
                m_stmNew.SetLength(0); // trucate stream
                XmlTextWriter xtw = new XmlTextWriter(m_stmNew,new UTF8Encoding());
                xdc.WriteContentTo(xtw);
                xtw.Flush();
            } // try
            catch (Exception)
            {
                // Do nothing, just pass encrypted value through...
            } // catch

            m_stmNew.Position = 0;
		}

		void Copy(Stream stmFrom, Stream stmTo)
		{
			TextReader trFrom = new StreamReader(stmFrom);
			TextWriter twTo = new StreamWriter(stmTo);
			twTo.WriteLine(trFrom.ReadToEnd());
			twTo.Flush();
		}
	}

	/// <summary>
	/// Summary description for Service1.
	/// </summary>
	public class Service1 : System.Web.Services.WebService
	{
		public Service1()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}
		#endregion

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
		}

		[WebMethod]
		[SoapDecrypter()]
		public string EchoTest(string strEchoMe)
		{
			return strEchoMe;
		}
	}
}
